@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2016  Mike Tzou
+ * Copyright (C) 2016-2023  Mike Tzou (Chocobo1)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,31 +29,26 @@
 #pragma once
 
 #include <type_traits>
-#include <QtGlobal>
+
+#include <QtProcessorDetection>
+
+#include <QString>
 
 #if (QT_POINTER_SIZE == 8)
 #define QBT_APP_64BIT
 #endif
 
-inline const int MAX_TORRENT_SIZE = 100 * 1024 * 1024; // 100 MiB
-
 template <typename T>
 constexpr typename std::add_const_t<T> &asConst(T &t) noexcept { return t; }
 
 // Forward rvalue as const
 template <typename T>
-constexpr typename std::add_const_t<T> asConst(T &&t) noexcept { return std::move(t); }
+constexpr typename std::add_const_t<T> asConst(T &&t) noexcept { return std::forward<T>(t); }
 
 // Prevent const rvalue arguments
 template <typename T>
 void asConst(const T &&) = delete;
 
-#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-// https://doc.qt.io/qt-6/qstring.html#operator-22-22_qs
-inline QString operator"" _qs(const char16_t *str, const std::size_t size)
-{
-    return QString::fromRawData(reinterpret_cast<const QChar *>(str), static_cast<int>(size));
-}
-#endif
+using namespace Qt::Literals::StringLiterals;
 
-inline const QString TORRENT_FILE_EXTENSION = u".torrent"_qs;
+inline const QString TORRENT_FILE_EXTENSION = u".torrent"_s;
