@@ -52,7 +52,7 @@ public:
 
 private:
     // scale bitfield vector to float vector
-    QVector<float> bitfieldToFloatVector(const QBitArray &vecin, int reqSize);
+    QList<float> bitfieldToFloatVector(const QBitArray &vecin, int reqSize);
     QImage renderImage() override;
     QString simpleToolTipText() const override;
     void updateColors() override;
