@@ -45,7 +45,7 @@
 StatsDialog::StatsDialog(QWidget *parent)
     : QDialog(parent)
     , m_ui(new Ui::StatsDialog)
-    , m_storeDialogSize(SETTINGS_KEY(u"Size"_qs))
+    , m_storeDialogSize(SETTINGS_KEY(u"Size"_s))
 {
     m_ui->setupUi(this);
 
@@ -86,13 +86,13 @@ void StatsDialog::update()
     m_ui->labelGlobalRatio->setText(
                 ((atd > 0) && (atu > 0))
                 ? Utils::String::fromDouble(static_cast<qreal>(atu) / atd, 2)
-                : u"-"_qs);
+                : u"-"_s);
 #ifndef QBT_USES_LIBTORRENT2
     // Cache hits
     const qreal readRatio = cs.readRatio;
-    m_ui->labelCacheHits->setText(u"%1%"_qs.arg((readRatio > 0)
+    m_ui->labelCacheHits->setText(u"%1%"_s.arg((readRatio > 0)
         ? Utils::String::fromDouble((100 * readRatio), 2)
-        : u"0"_qs));
+        : u"0"_s));
 #endif
     // Buffers size
     m_ui->labelTotalBuf->setText(Utils::Misc::friendlyUnit(cs.totalUsedBuffers * 16 * 1024));
@@ -100,12 +100,12 @@ void StatsDialog::update()
     // From lt manual: disk_write_queue and disk_read_queue are the number of peers currently waiting on a disk write or disk read
     // to complete before it receives or sends any more data on the socket. It's a metric of how disk bound you are.
 
-    m_ui->labelWriteStarve->setText(u"%1%"_qs.arg(((ss.diskWriteQueue > 0) && (ss.peersCount > 0))
+    m_ui->labelWriteStarve->setText(u"%1%"_s.arg(((ss.diskWriteQueue > 0) && (ss.peersCount > 0))
         ? Utils::String::fromDouble((100. * ss.diskWriteQueue / ss.peersCount), 2)
-        : u"0"_qs));
-    m_ui->labelReadStarve->setText(u"%1%"_qs.arg(((ss.diskReadQueue > 0) && (ss.peersCount > 0))
+        : u"0"_s));
+    m_ui->labelReadStarve->setText(u"%1%"_s.arg(((ss.diskReadQueue > 0) && (ss.peersCount > 0))
         ? Utils::String::fromDouble((100. * ss.diskReadQueue / ss.peersCount), 2)
-        : u"0"_qs));
+        : u"0"_s));
 
     // Disk queues
     m_ui->labelQueuedJobs->setText(QString::number(cs.jobQueueLength));
