@@ -32,7 +32,7 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonValue>
-#include <QVector>
+#include <QList>
 
 #include "base/rss/rss_article.h"
 #include "base/rss/rss_autodownloader.h"
