@@ -28,16 +28,16 @@
 
 "use strict";
 
-if (window.qBittorrent === undefined)
-    window.qBittorrent = {};
-
-window.qBittorrent.ContextMenu = (function() {
-    const exports = function() {
+window.qBittorrent ??= {};
+window.qBittorrent.ContextMenu ??= (() => {
+    const exports = () => {
         return {
             ContextMenu: ContextMenu,
             TorrentsTableContextMenu: TorrentsTableContextMenu,
+            StatusesFilterContextMenu: StatusesFilterContextMenu,
             CategoriesFilterContextMenu: CategoriesFilterContextMenu,
             TagsFilterContextMenu: TagsFilterContextMenu,
+            TrackersFilterContextMenu: TrackersFilterContextMenu,
             SearchPluginsTableContextMenu: SearchPluginsTableContextMenu,
             RssFeedContextMenu: RssFeedContextMenu,
             RssArticleContextMenu: RssArticleContextMenu,
@@ -46,31 +46,24 @@ window.qBittorrent.ContextMenu = (function() {
     };
 
     let lastShownContextMenu = null;
-    const ContextMenu = new Class({
-        // implements
-        Implements: [Options, Events],
-
-        // options
-        options: {
-            actions: {},
-            menu: "menu_id",
-            stopEvent: true,
-            targets: "body",
-            offsets: {
-                x: 0,
-                y: 0
-            },
-            onShow: () => {},
-            onHide: () => {},
-            onClick: () => {},
-            fadeSpeed: 200,
-            touchTimer: 600
-        },
-
-        // initialization
-        initialize: function(options) {
-            // set options
-            this.setOptions(options);
+    class ContextMenu {
+        constructor(options) {
+            this.options = {
+                actions: {},
+                menu: "menu_id",
+                stopEvent: true,
+                targets: "body",
+                offsets: {
+                    x: 0,
+                    y: 0
+                },
+                onShow: () => {},
+                onHide: () => {},
+                onClick: () => {},
+                fadeSpeed: 200,
+                touchTimer: 600,
+                ...options
+            };
 
             // option diffs menu
             this.menu = $(this.options.menu);
@@ -79,42 +72,35 @@ window.qBittorrent.ContextMenu = (function() {
             this.fx = new Fx.Tween(this.menu, {
                 property: "opacity",
                 duration: this.options.fadeSpeed,
-                onComplete: function() {
-                    if (this.getStyle("opacity"))
-                        this.setStyle("visibility", "visible");
-                    else
-                        this.setStyle("visibility", "hidden");
-                }.bind(this.menu)
+                onComplete: () => {
+                    this.menu.style.visibility = (getComputedStyle(this.menu).opacity > 0) ? "visible" : "hidden";
+                }
             });
 
             // hide and begin the listener
             this.hide().startListener();
 
             // hide the menu
-            this.menu.setStyles({
-                "position": "absolute",
-                "top": "-900000px",
-                "display": "block"
-            });
-        },
+            this.menu.style.position = "absolute";
+            this.menu.style.top = "-900000px";
+            this.menu.style.display = "block";
+        }
 
-        adjustMenuPosition: function(e) {
+        adjustMenuPosition(e) {
             this.updateMenuItems();
 
             const scrollableMenuMaxHeight = document.documentElement.clientHeight * 0.75;
 
-            if (this.menu.hasClass("scrollableMenu"))
-                this.menu.setStyle("max-height", scrollableMenuMaxHeight);
+            if (this.menu.classList.contains("scrollableMenu"))
+                this.menu.style.maxHeight = `${scrollableMenuMaxHeight}px`;
 
             // draw the menu off-screen to know the menu dimensions
-            this.menu.setStyles({
-                left: "-999em",
-                top: "-999em"
-            });
+            this.menu.style.left = "-999em";
+            this.menu.style.top = "-999em";
 
             // position the menu
-            let xPosMenu = e.page.x + this.options.offsets.x;
-            let yPosMenu = e.page.y + this.options.offsets.y;
+            let xPosMenu = e.pageX + this.options.offsets.x;
+            let yPosMenu = e.pageY + this.options.offsets.y;
             if ((xPosMenu + this.menu.offsetWidth) > document.documentElement.clientWidth)
                 xPosMenu -= this.menu.offsetWidth;
             if ((yPosMenu + this.menu.offsetHeight) > document.documentElement.clientHeight)
@@ -123,19 +109,17 @@ window.qBittorrent.ContextMenu = (function() {
                 xPosMenu = 0;
             if (yPosMenu < 0)
                 yPosMenu = 0;
-            this.menu.setStyles({
-                left: xPosMenu,
-                top: yPosMenu,
-                position: "absolute",
-                "z-index": "2000"
-            });
+            this.menu.style.left = `${xPosMenu}px`;
+            this.menu.style.top = `${yPosMenu}px`;
+            this.menu.style.position = "absolute";
+            this.menu.style.zIndex = "2000";
 
             // position the sub-menu
             const uls = this.menu.getElementsByTagName("ul");
             for (let i = 0; i < uls.length; ++i) {
                 const ul = uls[i];
-                if (ul.hasClass("scrollableMenu"))
-                    ul.setStyle("max-height", scrollableMenuMaxHeight);
+                if (ul.classList.contains("scrollableMenu"))
+                    ul.style.maxHeight = `${scrollableMenuMaxHeight}px`;
                 const rectParent = ul.parentNode.getBoundingClientRect();
                 const xPosOrigin = rectParent.left;
                 const yPosOrigin = rectParent.bottom;
@@ -149,27 +133,25 @@ window.qBittorrent.ContextMenu = (function() {
                     xPos = 0;
                 if (yPos < 0)
                     yPos = 0;
-                ul.setStyles({
-                    "margin-left": xPos - xPosOrigin,
-                    "margin-top": yPos - yPosOrigin
-                });
+                ul.style.marginLeft = `${xPos - xPosOrigin}px`;
+                ul.style.marginTop = `${yPos - yPosOrigin}px`;
             }
-        },
+        }
 
-        setupEventListeners: function(elem) {
-            elem.addEvent("contextmenu", (e) => {
+        setupEventListeners(elem) {
+            elem.addEventListener("contextmenu", (e) => {
                 this.triggerMenu(e, elem);
             });
-            elem.addEvent("click", (e) => {
+            elem.addEventListener("click", (e) => {
                 this.hide();
             });
 
-            elem.addEvent("touchstart", (e) => {
+            elem.addEventListener("touchstart", (e) => {
                 this.hide();
                 this.touchStartAt = performance.now();
                 this.touchStartEvent = e;
-            });
-            elem.addEvent("touchend", (e) => {
+            }, { passive: true });
+            elem.addEventListener("touchend", (e) => {
                 const now = performance.now();
                 const touchStartAt = this.touchStartAt;
                 const touchStartEvent = this.touchStartEvent;
@@ -177,130 +159,167 @@ window.qBittorrent.ContextMenu = (function() {
                 this.touchStartAt = null;
                 this.touchStartEvent = null;
 
-                const isTargetUnchanged = (Math.abs(e.event.pageX - touchStartEvent.event.pageX) <= 10) && (Math.abs(e.event.pageY - touchStartEvent.event.pageY) <= 10);
+                const isTargetUnchanged = (Math.abs(e.changedTouches[0].pageX - touchStartEvent.changedTouches[0].pageX) <= 10) && (Math.abs(e.changedTouches[0].pageY - touchStartEvent.changedTouches[0].pageY) <= 10);
                 if (((now - touchStartAt) >= this.options.touchTimer) && isTargetUnchanged)
                     this.triggerMenu(touchStartEvent, elem);
-            });
-        },
+            }, { passive: true });
+        }
 
-        addTarget: function(t) {
+        addTarget(t) {
             if (t.hasEventListeners)
                 return;
 
             // prevent long press from selecting this text
-            t.style.setProperty("user-select", "none");
-            t.style.setProperty("-webkit-user-select", "none");
+            t.style.userSelect = "none";
             t.hasEventListeners = true;
-
             this.setupEventListeners(t);
-        },
+        }
 
-        triggerMenu: function(e, el) {
+        searchAndAddTargets() {
+            if (this.options.targets.length > 0)
+                document.querySelectorAll(this.options.targets).forEach((target) => { this.addTarget(target); });
+        }
+
+        triggerMenu(e, el) {
             if (this.options.disabled)
                 return;
 
             // prevent default, if told to
-            if (this.options.stopEvent)
-                e.stop();
+            if (this.options.stopEvent) {
+                e.preventDefault();
+                e.stopPropagation();
+            }
             // record this as the trigger
             this.options.element = $(el);
             this.adjustMenuPosition(e);
             // show the menu
             this.show();
-        },
+        }
 
         // get things started
-        startListener: function() {
+        startListener() {
             /* all elements */
-            $$(this.options.targets).each((el) => {
-                this.addTarget(el);
-            }, this);
+            this.searchAndAddTargets();
 
             /* menu items */
-            this.menu.getElements("a").each(function(item) {
-                item.addEvent("click", (e) => {
-                    e.preventDefault();
-                    if (!item.hasClass("disabled")) {
-                        this.execute(item.get("href").split("#")[1], $(this.options.element));
-                        this.fireEvent("click", [item, e]);
-                    }
-                });
-            }, this);
+            this.menu.addEventListener("click", (e) => {
+                const menuItem = e.target.closest("li");
+                if (!menuItem)
+                    return;
+
+                e.preventDefault();
+                if (!menuItem.classList.contains("disabled")) {
+                    const anchor = menuItem.firstElementChild;
+                    this.execute(anchor.href.split("#")[1], this.options.element);
+                    this.options.onClick.call(this, anchor, e);
+                }
+                else {
+                    e.stopPropagation();
+                }
+            });
 
             // hide on body click
-            $(document.body).addEvent("click", () => {
+            $(document.body).addEventListener("click", () => {
                 this.hide();
+                this.options.element = null;
             });
-        },
+        }
 
-        updateMenuItems: function() {},
+        updateMenuItems() {}
 
         // show menu
-        show: function(trigger) {
+        show(trigger) {
             if (lastShownContextMenu && (lastShownContextMenu !== this))
                 lastShownContextMenu.hide();
             this.fx.start(1);
-            this.fireEvent("show");
+            this.options.onShow.call(this);
             lastShownContextMenu = this;
             return this;
-        },
+        }
 
         // hide the menu
-        hide: function(trigger) {
+        hide(trigger) {
             if (lastShownContextMenu && (lastShownContextMenu.menu.style.visibility !== "hidden")) {
                 this.fx.start(0);
-                // this.menu.fade('out');
-                this.fireEvent("hide");
+                this.options.onHide.call(this);
             }
             return this;
-        },
+        }
 
-        setItemChecked: function(item, checked) {
-            this.menu.getElement("a[href$=" + item + "]").firstChild.style.opacity =
+        setItemChecked(item, checked) {
+            this.menu.querySelector(`a[href$="${item}"]`).firstElementChild.style.opacity =
                 checked ? "1" : "0";
             return this;
-        },
+        }
 
-        getItemChecked: function(item) {
-            return this.menu.getElement("a[href$=" + item + "]").firstChild.style.opacity !== "0";
-        },
+        getItemChecked(item) {
+            return this.menu.querySelector(`a[href$="${item}"]`).firstElementChild.style.opacity !== "0";
+        }
 
         // hide an item
-        hideItem: function(item) {
-            this.menu.getElement("a[href$=" + item + "]").parentNode.addClass("invisible");
+        hideItem(item) {
+            this.menu.querySelector(`a[href$="${item}"]`).parentNode.classList.add("invisible");
             return this;
-        },
+        }
 
         // show an item
-        showItem: function(item) {
-            this.menu.getElement("a[href$=" + item + "]").parentNode.removeClass("invisible");
+        showItem(item) {
+            this.menu.querySelector(`a[href$="${item}"]`).parentNode.classList.remove("invisible");
             return this;
-        },
+        }
+
+        // enable/disable an item
+        setEnabled(item, enabled) {
+            this.menu.querySelector(`:scope a[href$="${item}"]`).parentElement.classList.toggle("disabled", !enabled);
+            return this;
+        }
 
         // disable the entire menu
-        disable: function() {
+        disable() {
             this.options.disabled = true;
             return this;
-        },
+        }
 
         // enable the entire menu
-        enable: function() {
+        enable() {
             this.options.disabled = false;
             return this;
-        },
+        }
 
         // execute an action
-        execute: function(action, element) {
+        execute(action, element) {
             if (this.options.actions[action])
                 this.options.actions[action](element, this, action);
             return this;
         }
-    });
+    };
 
-    const TorrentsTableContextMenu = new Class({
-        Extends: ContextMenu,
+    class FilterListContextMenu extends ContextMenu {
+        constructor(options) {
+            super(options);
+            this.torrentObserver = new MutationObserver((records, observer) => {
+                this.updateTorrentActions();
+            });
+        }
 
-        updateMenuItems: function() {
+        startTorrentObserver() {
+            this.torrentObserver.observe(torrentsTable.tableBody, { childList: true });
+        }
+
+        stopTorrentObserver() {
+            this.torrentObserver.disconnect();
+        }
+
+        updateTorrentActions() {
+            const torrentsVisible = torrentsTable.tableBody.children.length > 0;
+            this.setEnabled("startTorrents", torrentsVisible)
+                .setEnabled("stopTorrents", torrentsVisible)
+                .setEnabled("deleteTorrents", torrentsVisible);
+        }
+    };
+
+    class TorrentsTableContextMenu extends ContextMenu {
+        updateMenuItems() {
             let all_are_seq_dl = true;
             let there_are_seq_dl = false;
             let all_are_f_l_piece_prio = true;
@@ -313,11 +332,14 @@ window.qBittorrent.ContextMenu = (function() {
             let all_are_super_seeding = true;
             let all_are_auto_tmm = true;
             let there_are_auto_tmm = false;
+            let thereAreV1Hashes = false;
+            let thereAreV2Hashes = false;
             const tagCount = new Map();
+            const categoryCount = new Map();
 
             const selectedRows = torrentsTable.selectedRowsIds();
             selectedRows.forEach((item, index) => {
-                const data = torrentsTable.rows.get(item).full_data;
+                const data = torrentsTable.getRow(item).full_data;
 
                 if (data["seq_dl"] !== true)
                     all_are_seq_dl = false;
@@ -349,18 +371,29 @@ window.qBittorrent.ContextMenu = (function() {
                 else
                     all_are_auto_tmm = false;
 
+                if (data["infohash_v1"] !== "")
+                    thereAreV1Hashes = true;
+
+                if (data["infohash_v2"] !== "")
+                    thereAreV2Hashes = true;
+
                 const torrentTags = data["tags"].split(", ");
                 for (const tag of torrentTags) {
                     const count = tagCount.get(tag);
                     tagCount.set(tag, ((count !== undefined) ? (count + 1) : 1));
                 }
+
+                const torrentCategory = data["category"];
+                const count = categoryCount.get(torrentCategory);
+                categoryCount.set(torrentCategory, ((count !== undefined) ? (count + 1) : 1));
             });
 
             // hide renameFiles when more than 1 torrent is selected
             if (selectedRows.length === 1) {
-                const data = torrentsTable.rows.get(selectedRows[0]).full_data;
+                const data = torrentsTable.getRow(selectedRows[0]).full_data;
                 const metadata_downloaded = !((data["state"] === "metaDL") || (data["state"] === "forcedMetaDL") || (data["total_size"] === -1));
 
+                this.showItem("rename");
                 // hide renameFiles when metadata hasn't been downloaded yet
                 metadata_downloaded
                     ? this.showItem("renameFiles")
@@ -368,11 +401,12 @@ window.qBittorrent.ContextMenu = (function() {
             }
             else {
                 this.hideItem("renameFiles");
+                this.hideItem("rename");
             }
 
             if (all_are_downloaded) {
                 this.hideItem("downloadLimit");
-                this.menu.getElement("a[href$=uploadLimit]").parentNode.addClass("separator");
+                this.menu.querySelector("a[href$=uploadLimit]").parentNode.classList.add("separator");
                 this.hideItem("sequentialDownload");
                 this.hideItem("firstLastPiecePrio");
                 this.showItem("superSeeding");
@@ -382,10 +416,7 @@ window.qBittorrent.ContextMenu = (function() {
                 const show_seq_dl = (all_are_seq_dl || !there_are_seq_dl);
                 const show_f_l_piece_prio = (all_are_f_l_piece_prio || !there_are_f_l_piece_prio);
 
-                if (!show_seq_dl && show_f_l_piece_prio)
-                    this.menu.getElement("a[href$=firstLastPiecePrio]").parentNode.addClass("separator");
-                else
-                    this.menu.getElement("a[href$=firstLastPiecePrio]").parentNode.removeClass("separator");
+                this.menu.querySelector("a[href$=firstLastPiecePrio]").parentNode.classList.toggle("separator", (!show_seq_dl && show_f_l_piece_prio));
 
                 if (show_seq_dl)
                     this.showItem("sequentialDownload");
@@ -401,7 +432,7 @@ window.qBittorrent.ContextMenu = (function() {
                 this.setItemChecked("firstLastPiecePrio", all_are_f_l_piece_prio);
 
                 this.showItem("downloadLimit");
-                this.menu.getElement("a[href$=uploadLimit]").parentNode.removeClass("separator");
+                this.menu.querySelector("a[href$=uploadLimit]").parentNode.classList.remove("separator");
                 this.hideItem("superSeeding");
             }
 
@@ -423,91 +454,140 @@ window.qBittorrent.ContextMenu = (function() {
                 this.setItemChecked("autoTorrentManagement", all_are_auto_tmm);
             }
 
+            this.setEnabled("copyInfohash1", thereAreV1Hashes);
+            this.setEnabled("copyInfohash2", thereAreV2Hashes);
+
             const contextTagList = $("contextTagList");
-            tagList.forEach((tag, tagHash) => {
-                const checkbox = contextTagList.getElement(`a[href="#Tag/${tagHash}"] input[type="checkbox"]`);
-                const count = tagCount.get(tag.name);
+            for (const tag of tagMap.keys()) {
+                const checkbox = contextTagList.querySelector(`a[href="#Tag/${tag}"] input[type="checkbox"]`);
+                const count = tagCount.get(tag);
                 const hasCount = (count !== undefined);
                 const isLesser = (count < selectedRows.length);
                 checkbox.indeterminate = (hasCount ? isLesser : false);
                 checkbox.checked = (hasCount ? !isLesser : false);
-            });
-        },
-
-        updateCategoriesSubMenu: function(categoryList) {
-            const contextCategoryList = $("contextCategoryList");
-            contextCategoryList.getChildren().each(c => c.destroy());
-            contextCategoryList.appendChild(new Element("li", {
-                html: '<a href="javascript:torrentNewCategoryFN();"><img src="images/list-add.svg" alt="QBT_TR(New...)QBT_TR[CONTEXT=TransferListWidget]"/> QBT_TR(New...)QBT_TR[CONTEXT=TransferListWidget]</a>'
-            }));
-            contextCategoryList.appendChild(new Element("li", {
-                html: '<a href="javascript:torrentSetCategoryFN(0);"><img src="images/edit-clear.svg" alt="QBT_TR(Reset)QBT_TR[CONTEXT=TransferListWidget]"/> QBT_TR(Reset)QBT_TR[CONTEXT=TransferListWidget]</a>'
-            }));
-
-            const sortedCategories = [];
-            categoryList.forEach((category, hash) => sortedCategories.push({
-                categoryName: category.name,
-                categoryHash: hash
-            }));
-            sortedCategories.sort((left, right) => window.qBittorrent.Misc.naturalSortCollator.compare(
-                left.categoryName, right.categoryName));
-
-            let first = true;
-            for (const { categoryName, categoryHash } of sortedCategories) {
-                const el = new Element("li", {
-                    html: `<a href="javascript:torrentSetCategoryFN(${categoryHash});"><img src="images/view-categories.svg"/>${window.qBittorrent.Misc.escapeHtml(categoryName)}</a>`
-                });
-                if (first) {
-                    el.addClass("separator");
-                    first = false;
-                }
-                contextCategoryList.appendChild(el);
             }
-        },
 
-        updateTagsSubMenu: function(tagList) {
-            const contextTagList = $("contextTagList");
-            while (contextTagList.firstChild !== null)
-                contextTagList.removeChild(contextTagList.firstChild);
-
-            contextTagList.appendChild(new Element("li", {
-                html: '<a href="javascript:torrentAddTagsFN();">'
-                    + '<img src="images/list-add.svg" alt="QBT_TR(Add...)QBT_TR[CONTEXT=TransferListWidget]"/>'
-                    + " QBT_TR(Add...)QBT_TR[CONTEXT=TransferListWidget]"
-                    + "</a>"
-            }));
-            contextTagList.appendChild(new Element("li", {
-                html: '<a href="javascript:torrentRemoveAllTagsFN();">'
-                    + '<img src="images/edit-clear.svg" alt="QBT_TR(Remove All)QBT_TR[CONTEXT=TransferListWidget]"/>'
-                    + " QBT_TR(Remove All)QBT_TR[CONTEXT=TransferListWidget]"
-                    + "</a>"
-            }));
-
-            const sortedTags = [];
-            tagList.forEach((tag, hash) => sortedTags.push({
-                tagName: tag.name,
-                tagHash: hash
-            }));
-            sortedTags.sort((left, right) => window.qBittorrent.Misc.naturalSortCollator.compare(left.tagName, right.tagName));
-
-            for (let i = 0; i < sortedTags.length; ++i) {
-                const { tagName, tagHash } = sortedTags[i];
-                const el = new Element("li", {
-                    html: `<a href="#Tag/${tagHash}" onclick="event.preventDefault(); torrentSetTagsFN(${tagHash}, !event.currentTarget.getElement('input[type=checkbox]').checked);">`
-                        + '<input type="checkbox" onclick="this.checked = !this.checked;"> ' + window.qBittorrent.Misc.escapeHtml(tagName)
-                        + "</a>"
-                });
-                if (i === 0)
-                    el.addClass("separator");
-                contextTagList.appendChild(el);
+            const contextCategoryList = document.getElementById("contextCategoryList");
+            for (const category of categoryMap.keys()) {
+                const categoryIcon = contextCategoryList.querySelector(`a[href$="#Category/${category}"] img`);
+                const count = categoryCount.get(category);
+                const isEqual = ((count !== undefined) && (count === selectedRows.length));
+                categoryIcon.classList.toggle("highlightedCategoryIcon", isEqual);
             }
         }
-    });
 
-    const CategoriesFilterContextMenu = new Class({
-        Extends: ContextMenu,
-        updateMenuItems: function() {
-            const id = Number(this.options.element.id);
+        updateCategoriesSubMenu(categories) {
+            const contextCategoryList = $("contextCategoryList");
+            [...contextCategoryList.children].forEach((el) => { el.destroy(); });
+
+            const createMenuItem = (text, imgURL, clickFn) => {
+                const anchor = document.createElement("a");
+                anchor.textContent = text;
+                anchor.addEventListener("click", () => { clickFn(); });
+
+                const img = document.createElement("img");
+                img.src = imgURL;
+                img.alt = text;
+                anchor.prepend(img);
+
+                const item = document.createElement("li");
+                item.appendChild(anchor);
+
+                return item;
+            };
+            contextCategoryList.appendChild(createMenuItem("QBT_TR(New...)QBT_TR[CONTEXT=TransferListWidget]", "images/list-add.svg", torrentNewCategoryFN));
+            contextCategoryList.appendChild(createMenuItem("QBT_TR(Reset)QBT_TR[CONTEXT=TransferListWidget]", "images/edit-clear.svg", () => { torrentSetCategoryFN(""); }));
+
+            const sortedCategories = [...categories.keys()];
+            sortedCategories.sort(window.qBittorrent.Misc.naturalSortCollator.compare);
+
+            let first = true;
+            for (const categoryName of sortedCategories) {
+                const anchor = document.createElement("a");
+                anchor.href = `#Category/${categoryName}`;
+                anchor.textContent = categoryName;
+                anchor.addEventListener("click", (event) => {
+                    event.preventDefault();
+                    torrentSetCategoryFN(categoryName);
+                });
+
+                const img = document.createElement("img");
+                img.src = "images/view-categories.svg";
+                anchor.prepend(img);
+
+                const setCategoryItem = document.createElement("li");
+                setCategoryItem.appendChild(anchor);
+                if (first) {
+                    setCategoryItem.classList.add("separator");
+                    first = false;
+                }
+
+                contextCategoryList.appendChild(setCategoryItem);
+            }
+        }
+
+        updateTagsSubMenu(tags) {
+            const contextTagList = $("contextTagList");
+            contextTagList.replaceChildren();
+
+            const createMenuItem = (text, imgURL, clickFn) => {
+                const anchor = document.createElement("a");
+                anchor.textContent = text;
+                anchor.addEventListener("click", () => { clickFn(); });
+
+                const img = document.createElement("img");
+                img.src = imgURL;
+                img.alt = text;
+                anchor.prepend(img);
+
+                const item = document.createElement("li");
+                item.appendChild(anchor);
+
+                return item;
+            };
+            contextTagList.appendChild(createMenuItem("QBT_TR(Add...)QBT_TR[CONTEXT=TransferListWidget]", "images/list-add.svg", torrentAddTagsFN));
+            contextTagList.appendChild(createMenuItem("QBT_TR(Remove All)QBT_TR[CONTEXT=TransferListWidget]", "images/edit-clear.svg", torrentRemoveAllTagsFN));
+
+            const sortedTags = [...tags.keys()];
+            sortedTags.sort(window.qBittorrent.Misc.naturalSortCollator.compare);
+
+            for (let i = 0; i < sortedTags.length; ++i) {
+                const tagName = sortedTags[i];
+
+                const input = document.createElement("input");
+                input.type = "checkbox";
+                input.addEventListener("click", (event) => {
+                    input.checked = !input.checked;
+                });
+
+                const anchor = document.createElement("a");
+                anchor.href = `#Tag/${tagName}`;
+                anchor.textContent = tagName;
+                anchor.addEventListener("click", (event) => {
+                    event.preventDefault();
+                    torrentSetTagsFN(tagName, !input.checked);
+                });
+                anchor.prepend(input);
+
+                const setTagItem = document.createElement("li");
+                setTagItem.appendChild(anchor);
+                if (i === 0)
+                    setTagItem.classList.add("separator");
+
+                contextTagList.appendChild(setTagItem);
+            }
+        }
+    };
+
+    class StatusesFilterContextMenu extends FilterListContextMenu {
+        updateMenuItems() {
+            this.updateTorrentActions();
+        }
+    };
+
+    class CategoriesFilterContextMenu extends FilterListContextMenu {
+        updateMenuItems() {
+            const id = this.options.element.id;
             if ((id !== CATEGORIES_ALL) && (id !== CATEGORIES_UNCATEGORIZED)) {
                 this.showItem("editCategory");
                 this.showItem("deleteCategory");
@@ -521,52 +601,62 @@ window.qBittorrent.ContextMenu = (function() {
                 this.hideItem("deleteCategory");
                 this.hideItem("createSubcategory");
             }
-        }
-    });
 
-    const TagsFilterContextMenu = new Class({
-        Extends: ContextMenu,
-        updateMenuItems: function() {
-            const id = Number(this.options.element.id);
+            this.updateTorrentActions();
+        }
+    };
+
+    class TagsFilterContextMenu extends FilterListContextMenu {
+        updateMenuItems() {
+            const id = this.options.element.id;
             if ((id !== TAGS_ALL) && (id !== TAGS_UNTAGGED))
                 this.showItem("deleteTag");
             else
                 this.hideItem("deleteTag");
+
+            this.updateTorrentActions();
         }
-    });
+    };
 
-    const SearchPluginsTableContextMenu = new Class({
-        Extends: ContextMenu,
+    class TrackersFilterContextMenu extends FilterListContextMenu {
+        updateMenuItems() {
+            const id = this.options.element.id;
+            if ((id !== TRACKERS_ALL) && (id !== TRACKERS_TRACKERLESS))
+                this.showItem("deleteTracker");
+            else
+                this.hideItem("deleteTracker");
 
-        updateMenuItems: function() {
-            const enabledColumnIndex = function(text) {
-                const columns = $("searchPluginsTableFixedHeaderRow").getChildren("th");
-                for (let i = 0; i < columns.length; ++i) {
-                    if (columns[i].get("html") === "Enabled")
-                        return i;
-                }
+            this.updateTorrentActions();
+        }
+    };
+
+    class SearchPluginsTableContextMenu extends ContextMenu {
+        updateMenuItems() {
+            const enabledColumnIndex = (text) => {
+                const columns = document.querySelectorAll("#searchPluginsTableFixedHeaderRow th");
+                return Array.prototype.findIndex.call(columns, (column => column.textContent === "Enabled"));
             };
 
             this.showItem("Enabled");
-            this.setItemChecked("Enabled", this.options.element.getChildren("td")[enabledColumnIndex()].get("html") === "Yes");
+            this.setItemChecked("Enabled", (this.options.element.children[enabledColumnIndex()].textContent === "Yes"));
 
             this.showItem("Uninstall");
         }
-    });
+    };
 
-    const RssFeedContextMenu = new Class({
-        Extends: ContextMenu,
-        updateMenuItems: function() {
+    class RssFeedContextMenu extends ContextMenu {
+        updateMenuItems() {
             const selectedRows = window.qBittorrent.Rss.rssFeedTable.selectedRowsIds();
-            this.menu.getElement("a[href$=newSubscription]").parentNode.addClass("separator");
+            this.menu.querySelector("a[href$=newSubscription]").parentNode.classList.add("separator");
             switch (selectedRows.length) {
                 case 0:
                     // remove separator on top of newSubscription entry to avoid double line
-                    this.menu.getElement("a[href$=newSubscription]").parentNode.removeClass("separator");
+                    this.menu.querySelector("a[href$=newSubscription]").parentNode.classList.remove("separator");
                     // menu when nothing selected
                     this.hideItem("update");
                     this.hideItem("markRead");
                     this.hideItem("rename");
+                    this.hideItem("edit");
                     this.hideItem("delete");
                     this.showItem("newSubscription");
                     this.showItem("newFolder");
@@ -574,22 +664,24 @@ window.qBittorrent.ContextMenu = (function() {
                     this.hideItem("copyFeedURL");
                     break;
                 case 1:
-                    if (selectedRows[0] === 0) {
+                    if (selectedRows[0] === "0") {
                         // menu when "unread" feed selected
                         this.showItem("update");
                         this.showItem("markRead");
                         this.hideItem("rename");
+                        this.hideItem("edit");
                         this.hideItem("delete");
                         this.showItem("newSubscription");
                         this.hideItem("newFolder");
                         this.hideItem("updateAll");
                         this.hideItem("copyFeedURL");
                     }
-                    else if (window.qBittorrent.Rss.rssFeedTable.rows[selectedRows[0]].full_data.dataUid === "") {
+                    else if (window.qBittorrent.Rss.rssFeedTable.getRow(selectedRows[0]).full_data.dataUid === "") {
                         // menu when single folder selected
                         this.showItem("update");
                         this.showItem("markRead");
                         this.showItem("rename");
+                        this.hideItem("edit");
                         this.showItem("delete");
                         this.showItem("newSubscription");
                         this.showItem("newFolder");
@@ -601,6 +693,7 @@ window.qBittorrent.ContextMenu = (function() {
                         this.showItem("update");
                         this.showItem("markRead");
                         this.showItem("rename");
+                        this.showItem("edit");
                         this.showItem("delete");
                         this.showItem("newSubscription");
                         this.hideItem("newFolder");
@@ -613,6 +706,7 @@ window.qBittorrent.ContextMenu = (function() {
                     this.showItem("update");
                     this.showItem("markRead");
                     this.hideItem("rename");
+                    this.hideItem("edit");
                     this.showItem("delete");
                     this.hideItem("newSubscription");
                     this.hideItem("newFolder");
@@ -621,25 +715,20 @@ window.qBittorrent.ContextMenu = (function() {
                     break;
             }
         }
-    });
+    };
 
-    const RssArticleContextMenu = new Class({
-        Extends: ContextMenu
-    });
+    class RssArticleContextMenu extends ContextMenu {};
 
-    const RssDownloaderRuleContextMenu = new Class({
-        Extends: ContextMenu,
-        adjustMenuPosition: function(e) {
+    class RssDownloaderRuleContextMenu extends ContextMenu {
+        adjustMenuPosition(e) {
             this.updateMenuItems();
 
             // draw the menu off-screen to know the menu dimensions
-            this.menu.setStyles({
-                left: "-999em",
-                top: "-999em"
-            });
+            this.menu.style.left = "-999em";
+            this.menu.style.top = "-999em";
             // position the menu
-            let xPosMenu = e.page.x + this.options.offsets.x - $("rssdownloaderpage").offsetLeft;
-            let yPosMenu = e.page.y + this.options.offsets.y - $("rssdownloaderpage").offsetTop;
+            let xPosMenu = e.pageX + this.options.offsets.x - $("rssdownloaderpage").offsetLeft;
+            let yPosMenu = e.pageY + this.options.offsets.y - $("rssdownloaderpage").offsetTop;
             if ((xPosMenu + this.menu.offsetWidth) > document.documentElement.clientWidth)
                 xPosMenu -= this.menu.offsetWidth;
             if ((yPosMenu + this.menu.offsetHeight) > document.documentElement.clientHeight)
@@ -647,14 +736,12 @@ window.qBittorrent.ContextMenu = (function() {
             xPosMenu = Math.max(xPosMenu, 0);
             yPosMenu = Math.max(yPosMenu, 0);
 
-            this.menu.setStyles({
-                left: xPosMenu,
-                top: yPosMenu,
-                position: "absolute",
-                "z-index": "2000"
-            });
-        },
-        updateMenuItems: function() {
+            this.menu.style.left = `${xPosMenu}px`;
+            this.menu.style.top = `${yPosMenu}px`;
+            this.menu.style.position = "absolute";
+            this.menu.style.zIndex = "2000";
+        }
+        updateMenuItems() {
             const selectedRows = window.qBittorrent.RssDownloader.rssDownloaderRulesTable.selectedRowsIds();
             this.showItem("addRule");
             switch (selectedRows.length) {
@@ -678,9 +765,8 @@ window.qBittorrent.ContextMenu = (function() {
                     break;
             }
         }
-    });
+    };
 
     return exports();
 })();
-
 Object.freeze(window.qBittorrent.ContextMenu);
